from typing import Any, Dict, List, Optional

from whiskerrag_client.http_client import BaseClient
from whiskerrag_types.model.page import PageParams, PageResponse
from whiskerrag_types.model.task import Task


class TaskClient:
    def __init__(self, http_client: BaseClient):
        self.http_client = http_client
        self.base_path = "/api/task"

    async def get_task_list(
        self,
        page: int = 1,
        page_size: int = 10,
        order_by: Optional[str] = None,
        order_direction: str = "asc",
        eq_conditions: Optional[Dict[str, Any]] = None,
    ) -> PageResponse[Task]:
        params: PageParams = PageParams(
            page=page,
            page_size=page_size,
            order_by=order_by,
            order_direction=order_direction,
            eq_conditions=eq_conditions,
        )
        response = await self.http_client._request(
            method="POST",
            endpoint=f"{self.base_path}/list",
            json=params.model_dump(exclude_none=True),
        )
        return PageResponse(
            items=[Task(**task) for task in response["data"]["items"]],
            total=response["data"]["total"],
            page=response["data"]["page"],
            page_size=response["data"]["page_size"],
            total_pages=response["data"]["total_pages"],
        )

    async def get_task_detail(self, task_id: str) -> Task:
        response = await self.http_client._request(
            method="GET",
            endpoint=f"{self.base_path}/detail",
            params={"task_id": task_id},
        )
        return Task(**response["data"])

    async def restart_task(self, task_id_list: List[str]) -> List[Task]:
        response = await self.http_client._request(
            method="POST",
            endpoint=f"{self.base_path}/restart",
            json={"task_id_list": task_id_list},
        )

        return [Task(**task) for task in response["data"]]



from abc import ABC, abstractmethod
from typing import Any, List, TypeVar, Union

from pydantic import BaseModel

from whiskerrag_types.model.chunk import Chunk
from whiskerrag_types.model.retrieval import (
    RetrievalByKnowledgeRequest,
    RetrievalBySpaceRequest,
    RetrievalChunk,
)
from whiskerrag_types.model.space import Space

from ..model import Knowledge, PageParams, PageResponse, Task, Tenant
from .logger_interface import LoggerManagerInterface
from .settings_interface import SettingsInterface

T = TypeVar("T", bound=BaseModel)


class DBPluginInterface(ABC):
    settings: SettingsInterface
    logger: LoggerManagerInterface

    def __init__(
        self, logger: LoggerManagerInterface, settings: SettingsInterface
    ) -> None:
        self.settings = settings
        self.logger = logger
        self._initialized: bool = False

    async def ensure_initialized(self) -> None:
        if not self._initialized:
            try:
                self.logger.info("DBEngine plugin is initializing...")
                await self.init()
                self._initialized = True
                self.logger.info("DBEngine plugin is initialized")
            except Exception as e:
                self.logger.error(f"DBEngine plugin init error: {e}")
                raise

    @property
    def is_initialized(self) -> bool:
        return self._initialized

    @abstractmethod
    async def init(self) -> None:
        pass

    @abstractmethod
    async def cleanup(self) -> None:
        pass

    @abstractmethod
    def get_db_client(self) -> Any:
        pass

    # =================== knowledge ===================
    @abstractmethod
    async def save_knowledge_list(
        self, knowledge_list: List[Knowledge]
    ) -> List[Knowledge]:
        pass

    @abstractmethod
    async def get_knowledge_list(
        self, tenant_id: str, page_params: PageParams[Knowledge]
    ) -> PageResponse[Knowledge]:
        pass

    @abstractmethod
    async def get_knowledge(self, tenant_id: str, knowledge_id: str) -> Knowledge:
        pass

    @abstractmethod
    async def update_knowledge(self, knowledge: Knowledge) -> None:
        pass

    @abstractmethod
    async def delete_knowledge(
        self, tenant_id: str, knowledge_id_list: List[str]
    ) -> None:
        pass

    # =================== Space ===================
    @abstractmethod
    async def save_space(self, space: Space) -> Space:
        pass

    @abstractmethod
    async def update_space(self, space: Space) -> Space:
        pass

    @abstractmethod
    async def get_space_list(
        self, tenant_id: str, page_params: PageParams[Space]
    ) -> PageResponse[Space]:
        pass

    @abstractmethod
    async def get_space(self, tenant_id: str, space_id: str) -> Space:
        pass

    @abstractmethod
    async def delete_space(
        self, tenant_id: str, space_id: str
    ) -> Union[List[Space], None]:
        pass

    # =================== chunk ===================
    @abstractmethod
    async def save_chunk_list(self, chunks: List[Chunk]) -> List[Chunk]:
        pass

    @abstractmethod
    async def get_chunk_list(
        self, tenant_id: str, page_params: PageParams[Chunk]
    ) -> List[Chunk]:
        pass

    @abstractmethod
    async def get_chunk_by_id(self, tenant_id: str, chunk_id: str) -> Chunk:
        pass

    @abstractmethod
    async def delete_knowledge_chunk(
        self, tenant_id: str, knowledge_ids: List[str]
    ) -> Union[List[Chunk], None]:
        pass

    # =================== retrieval ===================
    @abstractmethod
    async def search_space_chunk_list(
        self,
        tenant_id: str,
        params: RetrievalBySpaceRequest,
    ) -> List[RetrievalChunk]:
        pass

    @abstractmethod
    async def search_knowledge_chunk_list(
        self,
        tenant_id: str,
        params: RetrievalByKnowledgeRequest,
    ) -> List[RetrievalChunk]:
        pass

    # =================== task ===================
    @abstractmethod
    async def save_task_list(self, task_list: List[Task]) -> List[Task]:
        pass

    @abstractmethod
    async def update_task_list(self, task_list: List[Task]) -> None:
        pass

    @abstractmethod
    async def get_task_list(
        self, tenant_id: str, page_params: PageParams[Task]
    ) -> PageResponse[Task]:
        pass

    @abstractmethod
    async def get_task_by_id(self, tenant_id: str, task_id: str) -> Union[Task, None]:
        pass

    @abstractmethod
    async def delete_knowledge_task(
        self, tenant_id: str, knowledge_ids: List[str]
    ) -> Union[List[Task], None]:
        pass

    # =================== tenant ===================
    @abstractmethod
    async def save_tenant(self, tenant: Tenant) -> Union[Tenant, None]:
        pass

    @abstractmethod
    async def get_tenant_by_sk(self, secret_key: str) -> Union[Tenant, None]:
        pass

    @abstractmethod
    async def update_tenant(self, tenant: Tenant) -> None:
        pass

    @abstractmethod
    async def validate_tenant_name(self, tenant_name: str) -> bool:
        pass

    @abstractmethod
    async def get_tenant_by_id(self, tenant_id: str) -> Union[Tenant, None]:
        pass

    @abstractmethod
    async def get_tenant_list(
        self, page_params: PageParams[Tenant]
    ) -> PageResponse[Tenant]:
        pass

    @abstractmethod
    async def delete_tenant_by_id(self, tenant_id: str) -> Union[Tenant, None]:
        pass



from typing import Literal, Optional, Union

from pydantic import BaseModel, Field, field_serializer, model_validator

from whiskerrag_types.model.knowledge import (
    EmbeddingModelEnum,
    GithubRepoSourceConfig,
    KnowledgeSourceEnum,
    KnowledgeTypeEnum,
    MetadataSerializer,
    OpenIdSourceConfig,
    OpenUrlSourceConfig,
    S3SourceConfig,
    TextSourceConfig,
)
from whiskerrag_types.model.splitter import (
    BaseCharSplitConfig,
    JSONSplitConfig,
    MarkdownSplitConfig,
    PDFSplitConfig,
    TextSplitConfig,
)


class KnowledgeCreateBase(BaseModel):
    space_id: str = Field(
        ...,
        description="the space of knowledge, example: petercat bot id, github repo name",
    )
    knowledge_type: KnowledgeTypeEnum = Field(
        KnowledgeTypeEnum.TEXT, description="type of knowledge resource"
    )
    knowledge_name: str = Field(
        ..., max_length=255, description="name of the knowledge resource"
    )
    metadata: dict = Field({}, description="additional metadata, user can update it")
    source_type: KnowledgeSourceEnum = Field(..., description="source type")
    embedding_model_name: Union[EmbeddingModelEnum, str] = Field(
        EmbeddingModelEnum.OPENAI,
        description="name of the embedding model. you can set any other model if target embedding service registered",
    )
    file_sha: Optional[str] = Field(None, description="SHA of the file")
    file_size: Optional[int] = Field(None, description="size of the file")

    @field_serializer("metadata")
    def serialize_metadata(self, metadata: dict) -> Optional[dict]:
        if metadata is None:
            return None
        sorted_metadata = MetadataSerializer.deep_sort_dict(metadata)
        return sorted_metadata if isinstance(sorted_metadata, dict) else None

    @field_serializer("knowledge_type")
    def serialize_knowledge_type(
        self, knowledge_type: Union[KnowledgeTypeEnum, str]
    ) -> str:
        if isinstance(knowledge_type, KnowledgeTypeEnum):
            return knowledge_type.value
        return str(knowledge_type)

    @field_serializer("source_type")
    def serialize_source_type(
        self, source_type: Union[KnowledgeSourceEnum, str]
    ) -> str:
        if isinstance(source_type, KnowledgeSourceEnum):
            return source_type.value
        return str(source_type)

    @field_serializer("embedding_model_name")
    def serialize_embedding_model_name(
        self, embedding_model_name: Union[EmbeddingModelEnum, str]
    ) -> str:
        if isinstance(embedding_model_name, EmbeddingModelEnum):
            return embedding_model_name.value
        return str(embedding_model_name)


class TextCreate(KnowledgeCreateBase):
    knowledge_type: Literal[KnowledgeTypeEnum.TEXT] = Field(
        KnowledgeTypeEnum.TEXT, description="type of knowledge resource"
    )
    source_config: Union[
        TextSourceConfig, OpenUrlSourceConfig, OpenIdSourceConfig, S3SourceConfig
    ] = Field(
        ...,
        description="source config of the knowledge",
    )
    split_config: TextSplitConfig = Field(
        ...,
        description="split config of the knowledge",
    )


class JSONCreate(KnowledgeCreateBase):
    knowledge_type: Literal[KnowledgeTypeEnum.JSON] = Field(
        KnowledgeTypeEnum.JSON, description="type of knowledge resource"
    )
    source_config: Union[
        TextSourceConfig, OpenUrlSourceConfig, OpenIdSourceConfig, S3SourceConfig
    ] = Field(
        ...,
        description="source config of the knowledge",
    )
    split_config: JSONSplitConfig = Field(
        ...,
        description="split config of the knowledge",
    )


class MarkdownCreate(KnowledgeCreateBase):
    knowledge_type: Literal[KnowledgeTypeEnum.MARKDOWN] = Field(
        KnowledgeTypeEnum.MARKDOWN, description="type of knowledge resource"
    )
    source_config: Union[
        TextSourceConfig, OpenUrlSourceConfig, OpenIdSourceConfig, S3SourceConfig
    ] = Field(
        ...,
        description="source config of the knowledge",
    )
    split_config: MarkdownSplitConfig = Field(
        ...,
        description="split config of the knowledge",
    )


class PDFCreate(KnowledgeCreateBase):
    knowledge_type: Literal[KnowledgeTypeEnum.PDF] = Field(
        KnowledgeTypeEnum.PDF, description="type of knowledge resource"
    )
    source_config: Union[OpenUrlSourceConfig, OpenIdSourceConfig, S3SourceConfig] = (
        Field(
            ...,
            description="source config of the knowledge",
        )
    )
    split_config: PDFSplitConfig = Field(
        ...,
        description="split config of the knowledge",
    )
    file_sha: str = Field(..., description="SHA of the file")
    file_size: int = Field(..., description="Byte size of the file")


class GithubRepoCreate(KnowledgeCreateBase):
    knowledge_type: Literal[KnowledgeTypeEnum.FOLDER] = Field(
        KnowledgeTypeEnum.FOLDER, description="type of knowledge resource"
    )
    source_config: GithubRepoSourceConfig = Field(
        ...,
        description="source config of the knowledge",
    )
    split_config: BaseCharSplitConfig = Field(
        ...,
        description="split config of the knowledge",
    )


class QACreate(KnowledgeCreateBase):
    knowledge_type: Literal[KnowledgeTypeEnum.QA] = Field(
        KnowledgeTypeEnum.QA, description="type of knowledge resource"
    )
    question: str = Field(..., description="question of the knowledge resource")
    answer: str = Field(..., description="answer of the knowledge resource")
    split_config: TextSplitConfig = Field(
        ...,
        description="split config of the knowledge",
    )
    source_config: Optional[TextSourceConfig] = Field(
        default=None,
        description="source config of the knowledge",
    )

    @model_validator(mode="after")
    def update_source_and_metadata(self) -> "QACreate":
        self.source_config = self.source_config or TextSourceConfig()
        self.source_config.text = self.question

        self.metadata = self.metadata or {}
        self.metadata["answer"] = self.answer

        return self


class ImageCreate(KnowledgeCreateBase):
    knowledge_type: Literal[KnowledgeTypeEnum.IMAGE] = Field(
        KnowledgeTypeEnum.IMAGE, description="type of knowledge resource"
    )
    source_config: Union[OpenUrlSourceConfig, OpenIdSourceConfig, S3SourceConfig] = (
        Field(
            ...,
            description="source config of the knowledge",
        )
    )
    file_sha: str = Field(..., description="SHA of the file")
    file_size: int = Field(..., description="Byte size of the file")


# TODO: add more knowledge types
# EXCEL = "excel"
# PPTX = "pptx"
# 定义一个类型变量来表示所有可能的知识创建类型
KnowledgeCreateUnion = Union[
    TextCreate,
    ImageCreate,
    JSONCreate,
    MarkdownCreate,
    PDFCreate,
    GithubRepoCreate,
    QACreate,
]



from whiskerrag_client.chunk_client import ChunkClient
from whiskerrag_client.http_client import HttpClient
from whiskerrag_client.knowledge_client import KnowledgeClient
from whiskerrag_client.retrieval_client import RetrievalClient
from whiskerrag_client.space_client import SpaceClient
from whiskerrag_client.task_client import TaskClient


class APIClient:

    def __init__(
        self,
        base_url: str,
        token: str,
        timeout: float = 10,
    ):
        self.http_client = HttpClient(base_url, token, timeout)
        self.knowledge = KnowledgeClient(self.http_client)
        self.retrieval = RetrievalClient(self.http_client)
        self.chunk = ChunkClient(self.http_client)
        self.task = TaskClient(self.http_client)
        self.space = SpaceClient(self.http_client)


__all__ = ["APIClient"]



from datetime import datetime, timezone
from typing import Any, List, Optional, Union
from uuid import UUID, uuid4

import numpy as np
from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    field_serializer,
    field_validator,
    model_validator,
)

from whiskerrag_types.model.knowledge import EmbeddingModelEnum
from whiskerrag_types.model.utils import parse_datetime


class Chunk(BaseModel):
    chunk_id: str = Field(default_factory=lambda: str(uuid4()), description="chunk id")
    space_id: str = Field(..., description="space id")
    tenant_id: str = Field(..., description="tenant id")
    embedding: Optional[list[float]] = Field(None, description="chunk embedding")
    context: str = Field(..., description="chunk content")
    knowledge_id: str = Field(..., description="file source info")
    embedding_model_name: Union[EmbeddingModelEnum, str] = Field(
        EmbeddingModelEnum.OPENAI, description="name of the embedding model"
    )
    metadata: Optional[dict] = Field(
        None, description="Arbitrary metadata associated with the content."
    )
    created_at: Optional[datetime] = Field(
        default=None,
        alias="gmt_create",
        description="creation time",
    )
    updated_at: Optional[datetime] = Field(
        default=None,
        alias="gmt_modified",
        description="update time",
    )

    @field_validator("embedding", mode="before")
    @classmethod
    def parse_embedding(cls, v: Union[str, List[float], None]) -> Optional[List[float]]:
        if v is None:
            return None
        if isinstance(v, np.ndarray):
            return v.tolist()

        if isinstance(v, list):
            return [float(x) for x in v]

        if isinstance(v, str):
            v = v.strip()
            try:
                import json

                return (
                    [float(x) for x in json.loads(v)]
                    if isinstance(json.loads(v), list)
                    else None
                )
            except json.JSONDecodeError:
                try:
                    if v.startswith("[") and v.endswith("]"):
                        v = v[1:-1]
                    return [float(x.strip()) for x in v.split(",") if x.strip()]
                except ValueError:
                    raise ValueError(f"Invalid embedding format: {v}")

        raise ValueError(f"Unsupported embedding type: {type(v)}")

    @field_serializer("embedding_model_name")
    def serialize_embedding_model_name(
        self, embedding_model_name: Union[EmbeddingModelEnum, str]
    ) -> str:
        if isinstance(embedding_model_name, EmbeddingModelEnum):
            return embedding_model_name.value
        return str(embedding_model_name)

    def update(self, **kwargs: Any) -> "Chunk":
        for key, value in kwargs.items():
            setattr(self, key, value)

        self.updated_at = datetime.now(timezone.utc)
        return self

    @model_validator(mode="before")
    def pre_process_data(cls, data: dict) -> dict:
        for field, value in data.items():
            if isinstance(value, UUID):
                data[field] = str(value)
        field_mappings = {"created_at": "gmt_create", "updated_at": "gmt_modified"}
        for field, alias_name in field_mappings.items():
            val = data.get(field) or data.get(alias_name)
            if val is None:
                continue

            if isinstance(val, str):
                dt = parse_datetime(val)
                data[field] = dt
                data[alias_name] = dt
            else:
                if val and val.tzinfo is None:
                    dt = val.replace(tzinfo=timezone.utc)
                    data[field] = dt
                    data[alias_name] = dt

        return data

    @model_validator(mode="after")
    def set_defaults(self) -> "Chunk":
        now = datetime.now(timezone.utc)
        if self.created_at is None:
            self.created_at = now
        if self.updated_at is None:
            self.updated_at = now
        return self

    model_config = ConfigDict(
        populate_by_name=True,
    )

    @field_serializer("created_at", "updated_at")
    def serialize_datetime(self, dt: datetime) -> str:
        return dt.strftime("%Y-%m-%dT%H:%M:%S.%fZ")
